import requests

BASE_URL = "http://127.0.0.1:8080"


def test_put(session):
    res = requests.put(f"{BASE_URL}/put")
    assert res.status_code == 200
    assert res.text == "PUT Request"


def test_put_with_param(session):
    res = requests.put(f"{BASE_URL}/put_with_body", data={"hello": "world"})

    assert res.status_code == 200
    assert res.text == "hello=world"



import requests

BASE_URL = "http://127.0.0.1:8080"


def test_delete(session):
    res = requests.delete(f"{BASE_URL}/delete")
    assert res.status_code == 200
    assert res.text == "DELETE Request"


def test_delete_with_param(session):
    res = requests.delete(f"{BASE_URL}/delete_with_body", data={"hello": "world"})
    assert res.status_code == 200
    assert res.text == "hello=world"



# from integration_tests.conftest import test_session
import os
import pathlib

import pytest

from robyn.env_populator import load_vars, parser

path = pathlib.Path(__file__).parent


# create robyn.env before test and delete it after test
@pytest.fixture
def env_file():
    CONTENT = """ROBYN_PORT=8081
    ROBYN_URL=127.0.0.1"""
    env_path = path / "robyn.env"
    env_path.write_text(CONTENT)
    yield
    env_path.unlink()
    os.unsetenv("ROBYN_PORT")
    os.unsetenv("ROBYN_URL")


# this tests if a connection can be made to the server with the correct port imported from the env file
def test_env_population(test_session, env_file):
    env_path = path / "robyn.env"
    load_vars(variables=parser(config_path=env_path))
    PORT = os.environ["ROBYN_PORT"]
    HOST = os.environ["ROBYN_URL"]
    assert PORT == "8080"
    assert HOST == "127.0.0.1"



import requests

BASE_URL = "http://127.0.0.1:8080"


def test_file_download_sync(session):
    r = requests.get(f"{BASE_URL}/file_download_sync")
    assert r.status_code == 200
    assert r.headers["Content-Disposition"] == "attachment"
    assert r.text == "This is a test file for the downloading purpose\n"


def test_file_download_async(session):
    r = requests.get(f"{BASE_URL}/file_download_async")
    assert r.status_code == 200
    assert r.headers["Content-Disposition"] == "attachment"
    assert r.text == "This is a test file for the downloading purpose\n"



import requests

BASE_URL = "http://127.0.0.1:8080"


def test_index_request(session):
    res = requests.get(f"{BASE_URL}")
    assert res.status_code == 200


def test_jsonify(session):
    r = requests.get(f"{BASE_URL}/jsonify")
    assert r.json() == {"hello": "world"}
    assert r.status_code == 200


def test_html(session):
    r = requests.get(f"{BASE_URL}/test/123")
    assert "Hello world. How are you?" in r.text


def test_jinja_template(session):
    r = requests.get(f"{BASE_URL}/template_render")
    assert "Jinja2" in r.text
    assert "Robyn" in r.text


def test_queries(session):
    r = requests.get(f"{BASE_URL}/query?hello=robyn")
    assert r.json() == {"hello": "robyn"}

    r = requests.get(f"{BASE_URL}/query")
    assert r.json() == {}


def test_request_headers(session):
    r = requests.get(f"{BASE_URL}/request_headers")
    assert r.status_code == 200
    assert r.text == "This is a regular response"
    assert "Header" in r.headers
    assert r.headers["Header"] == "header_value"


def test_const_request(session):
    r = requests.get(f"{BASE_URL}/const_request")
    assert "Hello world" in r.text
    assert r.status_code == 200


def test_const_request_json(session):
    r = requests.get(f"{BASE_URL}/const_request_json")
    assert r.status_code == 200
    assert r.json() == {"hello": "world"}


def test_const_request_headers(session):
    r = requests.get(f"{BASE_URL}/const_request_headers")
    assert r.status_code == 200
    assert "Header" in r.headers
    assert r.headers["Header"] == "header_value"


def test_response_type(session):
    r = requests.get(f"{BASE_URL}/types/response")
    assert r.status_code == 200
    assert r.text == "OK"


def test_str_type(session):
    r = requests.get(f"{BASE_URL}/types/str")
    assert r.status_code == 200
    assert r.text == "OK"


def test_int_type(session):
    r = requests.get(f"{BASE_URL}/types/int")
    assert r.status_code == 200
    assert r.text == "0"


def test_async_response_type(session):
    r = requests.get(f"{BASE_URL}/async/types/response")
    assert r.status_code == 200
    assert r.text == "OK"


def test_async_str_type(session):
    r = requests.get(f"{BASE_URL}/async/types/str")
    assert r.status_code == 200
    assert r.text == "OK"


def test_async_int_type(session):
    r = requests.get(f"{BASE_URL}/async/types/int")
    assert r.status_code == 200
    assert r.text == "0"



import requests

BASE_URL = "http://127.0.0.1:8080"


def test_post(session):
    res = requests.post(f"{BASE_URL}/post")
    assert res.status_code == 200
    assert res.text == "POST Request"


def test_post_with_param(session):
    res = requests.post(f"{BASE_URL}/post_with_body", data={"hello": "world"})
    assert res.text == "hello=world"
    assert res.status_code == 200


def test_jsonify_request(session):
    res = requests.post(f"{BASE_URL}/jsonify/123")
    assert res.status_code == 200
    assert res.json() == {"hello": "world"}


def test_post_request_headers(session):
    res = requests.post(f"{BASE_URL}/headers", headers={"hello": "world"})
    assert res.status_code == 200
    assert res.json()["hello"] == "world"



import requests

BASE_URL = "http://127.0.0.1:8080"


def test_patch(session):
    res = requests.patch(f"{BASE_URL}/patch")
    assert res.status_code == 200
    assert res.text == "PATCH Request"


def test_patch_with_param(session):
    res = requests.patch(f"{BASE_URL}/patch_with_body", data={"hello": "world"})
    assert res.status_code == 200
    assert res.text == "hello=world"



from __future__ import annotations

from dataclasses import dataclass
from typing import Callable, Optional

class SocketHeld:
    def __init__(self, url: str, port: int):
        pass
    def try_clone(self) -> SocketHeld:
        pass

@dataclass
class FunctionInfo:
    function: Callable
    is_async: bool
    number_of_params: int

@dataclass
class Response:
    status_code: int
    headers: dict[str, str]
    body: str

    def set_file_path(self, file_path: str):
        pass

class Server:
    def __init__(self) -> None:
        pass
    def add_directory(
        self,
        route: str,
        directory_path: str,
        index_file: Optional[str],
        show_files_listing: bool,
    ) -> None:
        pass
    def add_request_header(self, key: str, value: str) -> None:
        pass
    def add_route(
        self,
        route_type: str,
        route: str,
        function: FunctionInfo,
        is_const: bool,
    ) -> None:
        pass
    def add_middleware_route(
        self,
        route_type: str,
        route: str,
        function: FunctionInfo,
    ) -> None:
        pass
    def add_startup_handler(self, function: FunctionInfo) -> None:
        pass
    def add_shutdown_handler(self, function: FunctionInfo) -> None:
        pass
    def add_web_socket_route(
        self,
        route: str,
        connect_route: FunctionInfo,
        close_route: FunctionInfo,
        message_route: FunctionInfo,
    ) -> None:
        pass
    def start(self, socket: SocketHeld, workers: int) -> None:
        pass



from abc import ABC, abstractmethod
from asyncio import iscoroutinefunction
from functools import wraps
from inspect import signature
from types import CoroutineType
from typing import Callable, Dict, List, Tuple, Union

from robyn.robyn import FunctionInfo, Response
from robyn.ws import WS

Route = Tuple[str, str, FunctionInfo, bool]
MiddlewareRoute = Tuple[str, str, FunctionInfo]


class BaseRouter(ABC):
    @abstractmethod
    def add_route(*args) -> Union[Callable, CoroutineType, WS]:
        ...


class Router(BaseRouter):
    def __init__(self) -> None:
        super().__init__()
        self.routes = []

    def _format_response(self, res):
        response = {}
        if type(res) == dict:
            status_code = res.get("status_code", 200)
            headers = res.get("headers", {"Content-Type": "text/plain"})
            body = res.get("body", "")

            if type(status_code) != int:
                status_code = int(status_code)  # status_code can potentially be string

            response = Response(status_code=status_code, headers=headers, body=body)
            file_path = res.get("file_path")
            if file_path is not None:
                response.set_file_path(file_path)
        elif type(res) == Response:
            response = res
        else:
            response = Response(
                status_code=200, headers={"Content-Type": "text/plain"}, body=str(res)
            )

        return response

    def add_route(
        self, route_type: str, endpoint: str, handler: Callable, is_const: bool
    ) -> Union[Callable, CoroutineType]:
        @wraps(handler)
        async def async_inner_handler(*args):
            response = self._format_response(await handler(*args))
            return response

        @wraps(handler)
        def inner_handler(*args):
            response = self._format_response(handler(*args))
            return response

        number_of_params = len(signature(handler).parameters)
        if iscoroutinefunction(handler):
            function = FunctionInfo(async_inner_handler, True, number_of_params)
            self.routes.append((route_type, endpoint, function, is_const))
            return async_inner_handler
        else:
            function = FunctionInfo(inner_handler, False, number_of_params)
            self.routes.append((route_type, endpoint, function, is_const))
            return inner_handler

    def get_routes(self) -> List[Route]:
        return self.routes


class MiddlewareRouter(BaseRouter):
    def __init__(self) -> None:
        super().__init__()
        self.routes = []

    def add_route(self, route_type: str, endpoint: str, handler: Callable) -> Callable:
        number_of_params = len(signature(handler).parameters)
        function = FunctionInfo(handler, iscoroutinefunction(handler), number_of_params)
        self.routes.append((route_type, endpoint, function))
        return handler

    # These inner function is basically a wrapper arround the closure(decorator)
    # being returned.
    # It takes in a handler and converts it in into a closure
    # and returns the arguments.
    # Arguments are returned as they could be modified by the middlewares.
    def add_after_request(self, endpoint: str) -> Callable[..., None]:
        def inner(handler):
            @wraps(handler)
            async def async_inner_handler(*args):
                await handler(*args)
                return args

            @wraps(handler)
            def inner_handler(*args):
                handler(*args)
                return args

            if iscoroutinefunction(handler):
                self.add_route("AFTER_REQUEST", endpoint, async_inner_handler)
            else:
                self.add_route("AFTER_REQUEST", endpoint, inner_handler)

        return inner

    def add_before_request(self, endpoint: str) -> Callable[..., None]:
        def inner(handler):
            @wraps(handler)
            async def async_inner_handler(*args):
                await handler(*args)
                return args

            @wraps(handler)
            def inner_handler(*args):
                handler(*args)
                return args

            if iscoroutinefunction(handler):
                self.add_route("BEFORE_REQUEST", endpoint, async_inner_handler)
            else:
                self.add_route("BEFORE_REQUEST", endpoint, inner_handler)

        return inner

    def get_routes(self) -> List[MiddlewareRoute]:
        return self.routes


class WebSocketRouter(BaseRouter):
    def __init__(self) -> None:
        super().__init__()
        self.routes = {}

    def add_route(self, endpoint: str, web_socket: WS) -> None:
        self.routes[endpoint] = web_socket

    def get_routes(self) -> Dict[str, WS]:
        return self.routes



from abc import ABC, abstractmethod

from jinja2 import Environment, FileSystemLoader


class TemplateInterface(ABC):
    def __init__(self):
        ...

    @abstractmethod
    def render_template(self, *args, **kwargs):
        ...


class JinjaTemplate(TemplateInterface):
    def __init__(self, directory, encoding="utf-8", followlinks=False):
        self.env = Environment(
            loader=FileSystemLoader(
                searchpath=directory, encoding=encoding, followlinks=followlinks
            )
        )

    def render_template(self, template_name, **kwargs):
        return self.env.get_template(template_name).render(**kwargs)


__all__ = ["TemplateInterface", "JinjaTemplate"]
